@@ -10,8 +10,10 @@ require "sentry/core_ext/object/deep_dup"
 require "sentry/utils/argument_checking_helper"
 require "sentry/utils/encoding_helper"
 require "sentry/utils/logging_helper"
+require "sentry/utils/sample_rand"
 require "sentry/configuration"
-require "sentry/logger"
+require "sentry/structured_logger"
+require "sentry/debug_structured_logger"
 require "sentry/event"
 require "sentry/error_event"
 require "sentry/transaction_event"
@@ -25,6 +27,7 @@ require "sentry/session_flusher"
 require "sentry/backpressure_monitor"
 require "sentry/cron/monitor_check_ins"
 require "sentry/metrics"
+require "sentry/vernier/profiler"
 
 [
   "sentry/rake",
@@ -41,14 +44,25 @@ module Sentry
 
   CAPTURED_SIGNATURE = :@__sentry_captured
 
-  LOGGER_PROGNAME = "sentry".freeze
+  LOGGER_PROGNAME = "sentry"
 
-  SENTRY_TRACE_HEADER_NAME = "sentry-trace".freeze
+  SENTRY_TRACE_HEADER_NAME = "sentry-trace"
 
-  BAGGAGE_HEADER_NAME = "baggage".freeze
+  BAGGAGE_HEADER_NAME = "baggage"
 
   THREAD_LOCAL = :sentry_hub
 
+  MUTEX = Mutex.new
+
+  GLOBALS = %i[
+    main_hub
+    logger
+    session_flusher
+    backpressure_monitor
+    metrics_aggregator
+    exception_locals_tp
+  ].freeze
+
   class << self
     # @!visibility private
     def exception_locals_tp
@@ -127,7 +141,7 @@ module Sentry
     # @param version [String] version of the integration
     def register_integration(name, version)
       if initialized?
-        logger.warn(LOGGER_PROGNAME) do
+        sdk_logger.warn(LOGGER_PROGNAME) do
           <<~MSG
             Integration '#{name}' is loaded after the SDK is initialized, which can cause unexpected behavior.  Please make sure all integrations are loaded before SDK initialization.
           MSG
@@ -211,6 +225,13 @@ module Sentry
       get_current_scope.set_context(*args)
     end
 
+    # @!method add_attachment
+    #   @!macro add_attachment
+    def add_attachment(**opts)
+      return unless initialized?
+      get_current_scope.add_attachment(**opts)
+    end
+
     ##### Main APIs #####
 
     # Initializes the SDK with given configuration.
@@ -218,10 +239,11 @@ module Sentry
     # @yieldparam config [Configuration]
     # @return [void]
     def init(&block)
-      config = Configuration.new
-      yield(config) if block_given?
+      config = Configuration.new(&block)
+
       config.detect_release
       apply_patches(config)
+      config.validate
       client = Client.new(config)
       scope = Scope.new(max_breadcrumbs: config.max_breadcrumbs)
       hub = Hub.new(client, scope)
@@ -267,8 +289,10 @@ module Sentry
 
       @background_worker.shutdown
 
-      @main_hub = nil
-      Thread.current.thread_variable_set(THREAD_LOCAL, nil)
+      MUTEX.synchronize do
+        @main_hub = nil
+        Thread.current.thread_variable_set(THREAD_LOCAL, nil)
+      end
     end
 
     # Returns true if the SDK is initialized.
@@ -295,6 +319,9 @@ module Sentry
     #
     # @return [Hub]
     def get_main_hub
+      MUTEX.synchronize { @main_hub }
+    rescue ThreadError
+      # In some rare cases this may be called in a trap context so we need to handle it gracefully
       @main_hub
     end
 
@@ -465,6 +492,26 @@ module Sentry
       get_current_hub.capture_check_in(slug, status, **options)
     end
 
+    # Captures a log event and sends it to Sentry via the currently active hub.
+    # This is the underlying method used by the StructuredLogger class.
+    #
+    # @param message [String] the log message
+    # @param [Hash] options Extra log event options
+    # @option options [Symbol] level The log level (:trace, :debug, :info, :warn, :error, :fatal)
+    # @option options [Integer] severity The severity number according to the Sentry Logs Protocol
+    # @option options [String] origin The origin of the log event (e.g., "auto.db.rails", "manual")
+    # @option options [Hash] Additional attributes to include with the log
+    #
+    # @example Direct usage (prefer using Sentry.logger instead)
+    #   Sentry.capture_log("User logged in", level: :info, user_id: 123)
+    #
+    # @see https://develop.sentry.dev/sdk/telemetry/logs/ Sentry SDK Telemetry Logs Protocol
+    # @return [LogEvent, nil] The created log event or nil if logging is disabled
+    def capture_log(message, **options)
+      return unless initialized?
+      get_current_hub.capture_log_event(message, **options)
+    end
+
     # Takes or initializes a new Sentry::Transaction and makes a sampling decision for it.
     #
     # @return [Transaction, nil]
@@ -556,7 +603,7 @@ module Sentry
     #
     # @return [String]
     def get_trace_propagation_meta
-      return '' unless initialized?
+      return "" unless initialized?
       get_current_hub.get_trace_propagation_meta
     end
 
@@ -569,6 +616,45 @@ module Sentry
       get_current_hub.continue_trace(env, **options)
     end
 
+    # Returns the structured logger instance that implements Sentry's SDK telemetry logs protocol.
+    #
+    # This logger is only available when logs are enabled in the configuration.
+    #
+    # @example Enable logs in configuration
+    #   Sentry.init do |config|
+    #     config.dsn = "YOUR_DSN"
+    #     config.enable_logs = true
+    #   end
+    #
+    # @example Basic usage
+    #   Sentry.logger.info("User logged in successfully", user_id: 123)
+    #   Sentry.logger.error("Failed to process payment",
+    #     transaction_id: "tx_123",
+    #     error_code: "PAYMENT_FAILED"
+    #   )
+    #
+    # @see https://develop.sentry.dev/sdk/telemetry/logs/ Sentry SDK Telemetry Logs Protocol
+    #
+    # @return [StructuredLogger, nil] The structured logger instance or nil if logs are disabled
+    def logger
+      @logger ||=
+        if configuration.enable_logs
+          # Initialize the public-facing Structured Logger if logs are enabled
+          # Use configured structured logger class or default to StructuredLogger
+          # @see https://develop.sentry.dev/sdk/telemetry/logs/
+          configuration.structured_logging.logger_class.new(configuration)
+        else
+          warn <<~STR
+            [sentry] `Sentry.logger` will no longer be used as internal SDK logger when `enable_logs` feature is turned on.
+                    Use Sentry.configuration.sdk_logger for SDK-specific logging needs."
+
+                    Caller: #{caller.first}
+          STR
+
+          configuration.sdk_logger
+        end
+    end
+
     ##### Helpers #####
 
     # @!visibility private
@@ -580,8 +666,8 @@ module Sentry
     end
 
     # @!visibility private
-    def logger
-      configuration.logger
+    def sdk_logger
+      configuration.sdk_logger
     end
 
     # @!visibility private
@@ -593,6 +679,11 @@ module Sentry
     def utc_now
       Time.now.utc
     end
+
+    # @!visibility private
+    def dependency_installed?(name)
+      Object.const_defined?(name)
+    end
   end
 end
 
@@ -601,3 +692,6 @@ require "sentry/net/http"
 require "sentry/redis"
 require "sentry/puma"
 require "sentry/graphql"
+require "sentry/faraday"
+require "sentry/excon"
+require "sentry/std_lib_logger"
