@@ -12,11 +12,11 @@ module Sentry
       RUBY_INPUT_FORMAT = /
         ^ \s* (?: [a-zA-Z]: | uri:classloader: )? ([^:]+ | <.*>):
         (\d+)
-        (?: :in\s('|`)([^']+)')?$
-      /x.freeze
+        (?: :in\s('|`)(?:([\w:]+)\#)?([^']+)')?$
+      /x
 
       # org.jruby.runtime.callsite.CachingCallSite.call(CachingCallSite.java:170)
-      JAVA_INPUT_FORMAT = /^(.+)\.([^\.]+)\(([^\:]+)\:(\d+)\)$/.freeze
+      JAVA_INPUT_FORMAT = /^([\w$.]+)\.([\w$]+)\(([\w$.]+):(\d+)\)$/
 
       # The file portion of the line (such as app/models/user.rb)
       attr_reader :file
@@ -37,10 +37,11 @@ module Sentry
       # @return [Line] The parsed backtrace line
       def self.parse(unparsed_line, in_app_pattern = nil)
         ruby_match = unparsed_line.match(RUBY_INPUT_FORMAT)
+
         if ruby_match
-          _, file, number, _, method = ruby_match.to_a
+          _, file, number, _, module_name, method = ruby_match.to_a
           file.sub!(/\.class$/, RB_EXTENSION)
-          module_name = nil
+          module_name = module_name
         else
           java_match = unparsed_line.match(JAVA_INPUT_FORMAT)
           _, module_name, method, file, number = java_match.to_a
@@ -80,8 +81,6 @@ module Sentry
       end
     end
 
-    APP_DIRS_PATTERN = /(bin|exe|app|config|lib|test|spec)/.freeze
-
     # holder for an Array of Backtrace::Line instances
     attr_reader :lines
 
@@ -91,7 +90,7 @@ module Sentry
       ruby_lines = backtrace_cleanup_callback.call(ruby_lines) if backtrace_cleanup_callback
 
       in_app_pattern ||= begin
-        Regexp.new("^(#{project_root}/)?#{app_dirs_pattern || APP_DIRS_PATTERN}")
+        Regexp.new("^(#{project_root}/)?#{app_dirs_pattern}")
       end
 
       lines = ruby_lines.to_a.map do |unparsed_line|
